@@ -5,10 +5,6 @@ package Image::Info::JPEG;
 # This library is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
-# maintained by Tels 2007 - 2008
-
-$VERSION = 0.07;
-
 =begin register
 
 MAGIC: /^\xFF\xD8/
@@ -16,16 +12,12 @@ MAGIC: /^\xFF\xD8/
 For JPEG files we extract information both from C<JFIF> and C<Exif>
 application chunks.
 
-C<Exif> is the file format written by most digital cameras. This
+C<Exif> is the file format written by most digital cameras.  This
 encode things like timestamp, camera model, focal length, exposure
-time, aperture, flash usage, GPS position, etc.
+time, aperture, flash usage, GPS position, etc.  The following web
+page contain description of the fields that can be present:
 
-The C<Exif> spec can be found at:
-L<http://www.exif.org/specifications.html>.
-
-The C<color_type> element may have the following values: C<Gray>,
-C<YCbCr>, and C<CMYK>. Note that detecting C<RGB> and C<YCCK>
-currently does not work, but will hopefully in future.
+ http://www.butaman.ne.jp/~tsuruzoh/Computer/Digicams/exif-e.html
 
 =end register
 
@@ -55,73 +47,25 @@ sub my_read
     my $buf;
     my $n = read($source, $buf, $len);
     die "read failed: $!" unless defined $n;
-    die "short read ($len/$n) at pos " . tell($source) unless $n == $len;
+    die "short read ($len/$n)" unless $n == $len;
     $buf;
 }
 
-BEGIN {
-    my $f = ($] >= 5.008) ? <<'EOT' : <<'EOT';
-        sub with_io_string (&$) {
-	    open(my $fh, "<", \$_[1]);
-	    local $_ = $fh;
-	    &{$_[0]};
-	}
-EOT
-	sub with_io_string (&$) {
-	    require IO::String;
-	    local $_ = IO::String->new($_[1]);
-	    &{$_[0]};
-	    $_->close;
-	}
-EOT
-
-    #print $f;
-    eval $f;
-    die $@ if $@;
-}
-
 sub process_file
-{
-    my($info, $fh, $cnf) = @_;
-    _process_file($info, $fh, 0);
-}
-
-sub _process_file
 {
     my($info, $fh, $img_no) = @_;
+    $img_no ||= 0;
 
     my $soi = my_read($fh, 2);
-    unless ($soi eq "\xFF\xD8") {
-	my $ofs = tell() - 2;
-	die "SOI missing in JPEG file at offset $ofs";
-    }
+    die "SOI missing" unless $soi eq "\xFF\xD8";
 
     $info->push_info($img_no, "file_media_type" => "image/jpeg");
     $info->push_info($img_no, "file_ext" => "jpg");
 
     while (1) {
-        my($ff, $mark) = unpack("CC", my_read($fh, 2));
-	last if !defined $ff;
-        if ($ff != 0xFF) {
-	    my $corrupt_bytes = 2;
-	    while(1) {
-		my($ff) = unpack("C", my_read($fh,1));
-		return if !defined $ff;
-		last if $ff == 0xFF;
-		$corrupt_bytes++;
-	    }
-	    $mark = unpack("C", my_read($fh,1));
-	    $info->push_info($img_no, "Warn", sprintf("Corrupt JPEG data, $corrupt_bytes extraneous bytes before marker 0x%02x", $mark));
-	}
-	if ($mark == 0xFF) {
-	    # JPEG markers can be padded with unlimited 0xFF's
-	    for (;;) {
-		($mark) = unpack("C", my_read($fh, 1));
-		last if $mark != 0xFF;
-            }
-	}
+        my($ff, $mark, $len) = unpack("CCn", my_read($fh, 4));
+        last if $ff != 0xFF;
         last if $mark == 0xDA || $mark == 0xD9;  # SOS/EOI
-	my($len) = unpack("n", my_read($fh, 2));
 	last if $len < 2;
         process_chunk($info, $img_no, $mark, my_read($fh, $len - 2));
     }
@@ -142,13 +86,8 @@ sub process_chunk
         my($precision, $height, $width, $num_comp) =
             unpack("CnnC", substr($data, 0, 6, ""));
 	$info->push_info($img_no, "JPEG_Type", $sof{$mark});
-
-	# fix bug #15167 by keeping the highest values
-	my $old_w = $info->get_info($img_no, "width") || -1;
-	my $old_h = $info->get_info($img_no, "height") || -1;
-
-	$info->replace_info($img_no, "width", $width) if $old_w < $width;
-	$info->replace_info($img_no, "height", $height) if $old_h < $height;
+	$info->push_info($img_no, "width", $width);
+	$info->push_info($img_no, "height", $height);
 
 	for (1..$num_comp) {
 	    $info->push_info($img_no, "BitsPerSample", $precision);
@@ -167,24 +106,17 @@ sub process_chunk
 	}
 
 	if (1) {
-	    my %comp_id_lookup = ( 1 => "Y",
-				   2 => "Cb",
-				   3 => "Cr",
-				   82 => "R",
-				   71 => "G",
-				   66 => "B" );
 	    while (length($data)) {
 		my($comp_id, $hv, $qtable) =
 		    unpack("CCC", substr($data, 0, 3, ""));
-		my $horiz_sf = $hv >> 4 & 0x0f;
-		my $vert_sf = $hv & 0x0f;
-		$comp_id = $comp_id_lookup{$comp_id} || $comp_id;
-		$info->push_info($img_no, "ColorComponents",  [$comp_id, $hv, $qtable]);
-		$info->push_info($img_no, "ColorComponentsDecoded", 
-				 { ComponentIdentifier => $comp_id, 
-				   HorizontalSamplingFactor => $horiz_sf, 
-				   VerticalSamplingFactor => $vert_sf, 
-				   QuantizationTableDesignator => $qtable } );
+		$comp_id = { 1 => "Y",
+			     2 => "Cb",
+			     3 => "Cr",
+			     82 => "R",
+			     71 => "G",
+			     66 => "B",
+			   }->{$comp_id} || $comp_id;
+		$info->push_info(0, "ColorComponents", [$comp_id, $hv, $qtable]);
 	    }
 	}
     }
@@ -253,13 +185,13 @@ sub process_app0_jfxx
 		     { 0x10 => "JPEG thumbnail",
 		       0x11 => "Bitmap thumbnail",
 		       0x13 => "RGB thumbnail",
-		     }->{$code} || "Unknown extension code $code");
+		     }->{$code} || "Unknown extention code $code");
 
     if ($code == 0x10) {
 	eval {
-	    with_io_string {
-		_process_file($info, $_, 1);
-	    } $data;
+	    require IO::String;
+	    my $thumb_fh = IO::String->new($data);
+	    process_file($info, $thumb_fh, 1);
 	};
 	$info->push_info(1, "error" => $@) if $@;
     }
@@ -275,59 +207,31 @@ sub process_app1_exif
     }
 
     require Image::TIFF;
-    my $t = eval { Image::TIFF->new(\$data) };
-    if (!$t) {
-	$info->push_info(0, "Warn", "Cannot parse APP1 EXIF segment");
-	return;
-    }
-
+    my $t = Image::TIFF->new(\$data);
 
     for my $i (0 .. $t->num_ifds - 1) {
 	my $ifd = $t->ifd($i);
-
-#	use Data::Dumper;
-#	print STDERR Dumper($ifd);
-
 	for (@$ifd) {
-#		use Devel::Peek;
-#		print STDERR "# pushing info $i $_->[0] $_->[3]\n";
-#		print STDERR Devel::Peek::Dump($_->[3]),"\n" if $_->[0] =~ /Olympus-/;
-
 	    $info->push_info($i, $_->[0], $_->[3]);
 	}
 
 	# If we find JPEGInterchangeFormat/JPEGInterchangeFormatLngth,
-	# then we should apply process_file kind of recursively to extract
+	# then we should apply process_file kind of recusively to extract
 	# information of this (thumbnail) image file...
 	if (my($ipos) = $info->get_info($i, "JPEGInterchangeFormat", 1)) {
-	    my($ilen) = $info->get_info($i, "JPEGInterchangeFormatLength", 1);
-	    if ($ilen)
-		{
-		my $jdata = substr($data, $ipos, $ilen);
-		#$info->push_info($i, "JPEGImage" => $jdata);
+	    my($ilen) = $info->get_info($i, "JPEGInterchangeFormatLngth", 1);
+	    die unless $ilen;
+	    my $jdata = substr($data, $ipos, $ilen);
+	    #$info->push_info($i, "JPEGImage" => $jdata);
 
-		if ($jdata) {
-		    eval {
-			with_io_string {
-			    _process_file($info, $_, $i);
-			} $jdata;
-		    };
-		    $info->push_info($i, "error" => $@) if $@;
-		}
-	    }
+	    require IO::String;
+	    my $fh = IO::String->new($jdata);
+	    process_file($info, $fh, $i);
 	}
 
 	# Turn XResolution/YResolution into 'resolution'
 	my($xres) = $info->get_info($i, "XResolution", 1);
 	my($yres) = $info->get_info($i, "YResolution", 1);
-
-	# Samsung Digimax 200 is a totally confused camera that
-	# puts rational numbers with 0 as denominator and they
-	# also seem to not understand what resolution means.
-	for ($xres, $yres) {
-	    $_ += 0 if ref($_) eq "Image::TIFF::Rational";
-	}
-
 	my($unit) = $info->get_info($i, "ResolutionUnit", 1);
 	my $res = "1/1";  # default;
 	if ($xres && $yres) {
